@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2010 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -187,7 +187,7 @@ static void gkrellmd_syslog_log(GLogLevelFlags log_level, const gchar *message)
 	if (log_level & G_LOG_LEVEL_CRITICAL)
 		facility_priority = LOG_MAKEPRI(LOG_DAEMON, LOG_CRIT);
 
-	syslog(facility_priority, message);
+	syslog(facility_priority, "%s", message);
 #endif // defined(WIN32)
 	} // gkrellmd_syslog_log()
 
@@ -360,6 +360,7 @@ cidr_match(struct sockaddr *sa, socklen_t salen, char *allowed)
 	plen = -1;
 	if ((p = strchr(buf, '/')) != NULL)
 		{
+		errno = 0;
 		plen = strtoul(p + 1, &ep, 10);
 		if (errno != 0 || ep == NULL || *ep != '\0' || plen < 0)
 			{
@@ -787,17 +788,14 @@ read_config(void)
 
 // on windows also load config from INSTALLDIR/etc/gkrellmd.conf
 #if defined(WIN32)
-#if GLIB_CHECK_VERSION(2,16,0)
 	install_path = g_win32_get_package_installation_directory_of_module(NULL);
-	path = g_build_filename(install_path, "etc", GKRELLMD_CONFIG, NULL);
-#else
-	// deprecated since glib 2.16
-	install_path = g_win32_get_package_installation_subdirectory(NULL, NULL, "etc");
-	path = g_build_filename(install_path, GKRELLMD_CONFIG, NULL);
-#endif
-	load_config(path);
-	g_free(install_path);
-	g_free(path);
+	if (install_path != NULL)
+		{
+		path = g_build_filename(install_path, "etc", GKRELLMD_CONFIG, NULL);
+		load_config(path);
+		g_free(path);
+		g_free(install_path);
+		}
 #endif
 
 	_GK.homedir = (gchar *) g_get_home_dir();
@@ -1003,6 +1001,13 @@ socksetup(int af)
 #endif
 				continue;
 				}
+
+#ifdef IP_FREEBIND
+			if (setsockopt(*s, SOL_IP, IP_FREEBIND, &on, sizeof(on)) < 0)
+				{
+				g_warning("gkrellmd: setsockopt (IP_FREEBIND) failed\n");
+				}
+#endif
 			}
 
 #ifdef IPV6_V6ONLY
@@ -1098,7 +1103,12 @@ detach_from_terminal(void)
 		if (fd > 2)
 			close(fd);
 		}
-	chdir("/");
+
+	if (chdir("/") != 0)
+		{
+		g_warning("Detach failed in chdir(\"/\"): %s\n", strerror(errno));
+		return FALSE;
+		}
 #endif /* HAVE_DAEMON */
 
 //	signal(SIGCHLD, SIG_IGN);
@@ -1193,10 +1203,6 @@ gkrellmd_run(gint argc, gchar **argv)
 	gkrellm_sys_main_init();
 	drop_privileges();
 
-#if GLIB_CHECK_VERSION(2,0,0)
-	g_thread_init(NULL);
-#endif
-
 	_GK.start_time = time(0);
 	if (_GK.update_HZ < 1 || _GK.update_HZ > 10)
 		_GK.update_HZ = 3;
@@ -1313,8 +1319,9 @@ gkrellmd_run(gint argc, gchar **argv)
 				if (!client)
 					{
 #if defined(WIN32)
-				closesocket(client_fd);
+					closesocket(client_fd);
 #else
+					shutdown(client_fd, SHUT_WR);
 					close(client_fd);
 #endif
 					continue;
@@ -1606,7 +1613,7 @@ static gboolean service_uninstall()
 		}
 	else
 		{
-		// handle to gkrellm service aquired, now stop and uninstall it
+		// handle to gkrellm service acquired, now stop and uninstall it
 		if (service_stop(svcHandle))
 			{
 			delRet = DeleteService(svcHandle);
@@ -1660,11 +1667,8 @@ int main(int argc, char* argv[])
 	if (install_path != NULL)
 		{
 	    locale_dir = g_build_filename(install_path, LOCALEDIR, NULL);
-		if (locale_dir != NULL)
-			{
-			bindtextdomain(PACKAGE_D, locale_dir);
-			g_free(locale_dir);
-			}
+		bindtextdomain(PACKAGE_D, locale_dir);
+		g_free(locale_dir);
 	    g_free(install_path);
 		}
 #else
