@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2010 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -284,23 +284,14 @@ gkrellmd_plugins_load(void)
 	g_free(path);
 
 #if defined(WIN32)
-	path = NULL;
-#if GLIB_CHECK_VERSION(2,16,0)
 	gchar *install_path;
 	install_path = g_win32_get_package_installation_directory_of_module(NULL);
 	if (install_path != NULL)
 		{
 		path = g_build_filename(install_path, "lib", "gkrellm2", "plugins-gkrellmd", NULL);
-		g_free(install_path);
-		}
-#else
-	// deprecated since glib 2.16.0
-	path = g_win32_get_package_installation_subdirectory(NULL, NULL, "lib/gkrellm2/plugins-gkrellmd");
-#endif
-	if (path)
-		{
 		gkrellmd_plugin_scan(path);
 		g_free(path);
+		g_free(install_path);
 		}
 #endif
 
