@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2009 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -171,11 +171,11 @@ read_battery_demo(void)
 static gboolean
 setup_battery_interface(void)
 	{
-    if (!read_battery_data && !_GK.client_mode && gkrellm_sys_battery_init())
-        read_battery_data = gkrellm_sys_battery_read_data;
+	if (!read_battery_data && !_GK.client_mode && gkrellm_sys_battery_init())
+		read_battery_data = gkrellm_sys_battery_read_data;
 	if (_GK.demo)
 		read_battery_data = read_battery_demo;
-    return read_battery_data ? TRUE : FALSE;
+	return read_battery_data ? TRUE : FALSE;
 	}
 
 void 
@@ -534,8 +534,8 @@ update_battery(void)
 static gboolean
 cb_expose_event(GtkWidget *widget, GdkEventExpose *ev, GkrellmPanel *p)
 	{
-	gdk_draw_drawable(widget->window,
-			widget->style->fg_gc[GTK_WIDGET_STATE(widget)], p->pixmap,
+	gdk_draw_drawable(gtk_widget_get_window(widget),
+                          gtk_widget_get_style(widget)->fg_gc[gtk_widget_get_state(widget)], p->pixmap,
 			ev->area.x, ev->area.y, ev->area.x, ev->area.y,
 			ev->area.width, ev->area.height);
 	return FALSE;
