@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -255,6 +267,16 @@ gkrellm_set_chartdata_flags(GkrellmChartdata *cd, gint flags)
 		cd->flags = flags;
 	}
 
+static void
+gk_draw_vertical_line(GdkDrawable *drawable, GdkGC *gc, gint x, gint y1, gint y2)
+	{
+	if (y1 == y2)
+		gdk_draw_point(drawable, gc, x, y1);
+	else
+		gdk_draw_line(drawable, gc, x, y1, x, y2);
+	}
+
+
 static gint
 chartdata_ycoord(GkrellmChart *cp, GkrellmChartdata *cd, gint yd)
 	{
@@ -401,7 +423,7 @@ draw_chartdata_impulses(GkrellmChart *cp, GList *cd_list,
 				yI += cd->data[x];
 				y = chartdata_ycoord(cp, cd, yI);
 				if (cd->data[x] > 0)
-					gdk_draw_line(cd->data_bitmap, _GK.bit1_GC, n, y1, n, y);
+					gk_draw_vertical_line(cd->data_bitmap, _GK.bit1_GC, n, y1, y);
 				y1 = y;
 				}
 			else
@@ -411,7 +433,7 @@ draw_chartdata_impulses(GkrellmChart *cp, GList *cd_list,
 				yN += cd->data[x];
 				y = chartdata_ycoord(cp, cd, yN);
 				if (cd->data[x] > 0)
-					gdk_draw_line(cd->data_bitmap, _GK.bit1_GC, n, y0, n, y);
+					gk_draw_vertical_line(cd->data_bitmap, _GK.bit1_GC, n, y0, y);
 				y0 = y;
 				}
 			}
@@ -489,11 +511,7 @@ draw_chartdata_lines(GkrellmChart *cp, GkrellmChartdata *cd, gint i0)
 		cd->y_p_valid = FALSE;	/* Need a store_chartdata to make it valid */
 		if (cd->data[x] > 0 || (cd->inverted ? (y > y0) : (y < y0)))
 			{
-			if (y == y1)
-				gdk_draw_point(cd->data_bitmap, _GK.bit1_GC, cp->x + n, y1);
-			else
-				gdk_draw_line(cd->data_bitmap, _GK.bit1_GC,
-						cp->x + n, y, cp->x + n, y1);
+			gk_draw_vertical_line(cd->data_bitmap, _GK.bit1_GC, cp->x + n, y, y1);
 			}
 		}
 	/* Push the grided pixmap through the data bitmap onto the expose pixmap
@@ -680,7 +698,7 @@ gkrellm_alloc_chartdata(GkrellmChart *cp)
 		cd->w = w;
 		if (cd->data)
 			g_free(cd->data);
-		cd->data = (gint *) g_new0(gint, w);
+		cd->data = (gulong *) g_new0(gulong, w);
 		cd->maxval = 0;
 		cp->position = cp->w - 1;
 		cp->tail = cp->position;
@@ -797,6 +815,17 @@ void
 gkrellm_store_chartdata(GkrellmChart *cp, gulong total, ...)
 	{
 	va_list			args;
+
+	if (!cp)
+		return;
+	va_start(args, total);
+	gkrellm_store_chartdatav(cp, total, args);
+	va_end(args);
+	}
+
+void
+gkrellm_store_chartdatav(GkrellmChart *cp, gulong total, va_list args)
+	{
 	GList			*list;
 	GkrellmChartdata *cd;
 	gulong			range, total_diff;
@@ -806,10 +835,10 @@ gkrellm_store_chartdata(GkrellmChart *cp, gulong total, ...)
 
 	if (!cp)
 		return;
-	va_start(args, total);
 	for (list = cp->cd_list; list; list = list->next)
 		{
 		cd = (GkrellmChartdata *) list->data;
+		//FIXME: missing check for number of passed varargs passed in "args"
 		cd->current = va_arg(args, gulong);
 		if (!cd->monotonic)
 			{
@@ -818,10 +847,15 @@ gkrellm_store_chartdata(GkrellmChart *cp, gulong total, ...)
 			}
 		/* Prime the pump.  Also handle data wrap around or reset to zero.
 		*/
-		if (cd->current < cd->previous || !cp->primed)
+		cd->wrap = 0;
+		if (!cp->primed)
 			cd->previous = cd->current;
+		else if (cd->current < cd->previous)
+			{
+			cd->wrap = (gulong) -1 - cd->previous;
+			cd->previous = 0;
+			}
 		}
-	va_end(args);
 	if (total < cp->previous_total || !cp->primed)
 		cp->previous_total = total;	  /* Wrap around, this store won't scale */
 	total_diff = total - cp->previous_total;
@@ -843,7 +877,7 @@ gkrellm_store_chartdata(GkrellmChart *cp, gulong total, ...)
 		{
 		cd = (GkrellmChartdata *) list->data;
 		cd->discard = cd->data[cp->tail];
-		cd->data[n] = (gint)(cd->current - cd->previous);
+		cd->data[n] = (gulong)(cd->current - cd->previous) + cd->wrap;
 		cd->previous = cd->current;
 
 		/* If using totals, scale the stored data to range between 0 and the
@@ -892,7 +926,9 @@ gkrellm_store_chartdata(GkrellmChart *cp, gulong total, ...)
 				else if (N > cd->maxval)
 					cd->maxval = N;
 				}
-			if (N + I > cp->maxval)
+			if (N_discard + I_discard >= cd->maxval)
+				need_scan = TRUE;
+			else if (N + I > cp->maxval)
 				cp->maxval = N + I;
 			}
 		else if (cd->draw_style == CHARTDATA_LINE)
@@ -912,11 +948,7 @@ gkrellm_store_chartdata(GkrellmChart *cp, gulong total, ...)
 	cp->primed = TRUE;
 	if (need_scan || cp->redraw_all)
 		scan_for_maxval(cp);
-#if defined(WIN32)
-	cp->redraw_all = TRUE;
-#else
 	scroll_chartdata_bitmaps(cp);
-#endif
 	}
 
 
@@ -997,6 +1029,13 @@ gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 	   )
 		cp->text_format_reuse = TRUE;
 
+	if (_GK.debug_level & DEBUG_CHART_TEXT)
+		{
+		g_debug("\n");
+		if (!cp->text_format_reuse)
+			g_debug("draw_chart_text: [%s]\n", str);
+		}
+
 	if (   !cp->text_format_reuse
 		|| cp->text_run_sequence_id != cp->bg_sequence_id
 	   )
@@ -1086,9 +1125,9 @@ gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 				ts = ts_alt;
 				shadow = ts_alt->effect ? 1 : 0;
 				}
-			else if (c == 'x' && isdigit(*s))
+			else if (c == 'x' && isdigit((unsigned char)*s))
 				xx = *s++ - '0';
-			else if (c == 'y' && isdigit(*s))
+			else if (c == 'y' && isdigit((unsigned char)*s))
 				y = *s++ - '0';
 			else if (c == 'D')
 				{
@@ -1137,13 +1176,15 @@ gkrellm_draw_chart_text(GkrellmChart *cp, gint style_id, gchar *str)
 				tr->cache_valid = FALSE;
 			tr->x = x;
 			tr->y = y;
-#if 0
-gchar buf[128];
-strncpy(buf, s, text_length);
-buf[text_length] = '\0';
-if (!tr->cache_valid)
-printf("pango draw chart text: %s\n", buf);
-#endif
+
+			if (_GK.debug_level & DEBUG_CHART_TEXT)
+				{
+				gchar	buf[128];
+
+				strncpy(buf, s, text_length);
+				buf[text_length] = '\0';
+				g_debug("draw_chart_text: [%s] ", buf);
+				}
 			
 			offset = cp->baseline_ref - tr->baseline;	/* align baselines */
 			if (_GK.chart_text_no_fill)
@@ -1153,6 +1194,8 @@ printf("pango draw chart text: %s\n", buf);
 				{
 				if (!tr->cache_valid)
 					{
+					if (_GK.debug_level & DEBUG_CHART_TEXT)
+						g_debug("pango ");
 					gdk_draw_drawable(cp->bg_text_pixmap, _GK.draw1_GC,
 							cp->bg_pixmap,
 							x - 1, y + tr->y_ink + offset - 1,
@@ -1161,6 +1204,10 @@ printf("pango draw chart text: %s\n", buf);
 		    		gkrellm_draw_text(cp->bg_text_pixmap, ts, x, y + offset, s,
 							text_length);
 					}
+				if (_GK.debug_level & DEBUG_CHART_TEXT)
+					g_debug("x=%d y=%d w=%d h=%d\n",
+							x - 1, y + tr->y_ink + offset - 1,
+							tr->w + shadow + 2, tr->h + shadow + 1);
 				gdk_draw_drawable(cp->pixmap, _GK.draw1_GC, cp->bg_text_pixmap,
 						x - 1, y + tr->y_ink + offset - 1,
 						x - 1, y + tr->y_ink + offset - 1,
@@ -1417,7 +1464,7 @@ gkrellm_add_chartdata(GkrellmChart *cp, GdkPixmap **src_pixmap,
 	if (!cp || !src_pixmap || !grid_pixmap || !label)
 		return NULL;
 
-	/* To handle theme and vert size changes without loosing data, reuse the
+	/* To handle theme and vert size changes without losing data, reuse the
 	|  GkrellmChartdata structs in the cd_list.
 	*/
 	list = g_list_nth(cp->cd_list, cp->cd_list_index++);
@@ -1944,32 +1991,35 @@ gkrellm_125_sequence(gint value, gboolean use125,
 		return table[0];
 	if (value > table[table_size - 1])
 		return table[table_size - 1];
-	for (i = 0; i < table_size; ++i)
+	if (!snap_to_table && !roundup)
 		{
-/*printf("  mapping[%d] value=%d table=%d\n", i, value, table[i]); */
-		if (value == table[i])
-			return table[i];
-		else if (value == table[i] - 1)
-			return table[i - 1];
-		else if (value == table[i] + 1)
-			return table[i + 1];
+		for (i = 0; i < table_size; ++i)
+			{
+/*g_debug("  mapping[%d] value=%d table=%d\n", i, value, table[i]); */
+			if (value == table[i])
+				return table[i];
+			else if (value == table[i] - 1)
+				return table[i - 1];
+			else if (value == table[i] + 1)
+				return table[i + 1];
+			}
 		}
-	if (snap_to_table && !roundup)
+	else if (snap_to_table && !roundup)
 		{
 		for (i = table_size - 1; i >= 0; --i)
 			{
-			if (value > table[i])
+			if (value >= table[i])
 				{
 				value = table[i];
 				break;
 				}
 			}
 		}
-	if (snap_to_table && roundup)
+	else if (snap_to_table && roundup)
 		{
 		for (i = 0; i < table_size; ++i)
 			{
-			if (value < table[i])
+			if (value <= table[i])
 				{
 				value = table[i];
 				break;
@@ -1998,6 +2048,7 @@ cb_chart_grid_resolution(GtkWidget *adjustment, GkrellmChart *cp)
 	gint			res;
 	gfloat			fres;
 
+	_GK.config_modified = TRUE;
 	cf = cp->config;
 	spin = GTK_SPIN_BUTTON(cf->grid_resolution_spin_button);
 	if (cf->map_sequence)
@@ -2205,6 +2256,7 @@ gkrellm_set_chart_height(GkrellmChart *cp, gint h)
 		cd->data_bitmap = gdk_pixmap_new(top_win->window, cp->w, cp->h, 1);
 		cd->layer.pixmap = gdk_pixmap_new(top_win->window, cp->w, cp->h, -1);
 		}
+	cp->bg_sequence_id += 1;
 	render_chart_pixmaps(cp);
 
 	cf = cp->config;
@@ -2264,15 +2316,15 @@ set_resolution_menubar_items_sensitivity(GkrellmChartconfig *cf)
 	{
 	GtkWidget	*w;
 
-	if (!cf->auto_resolution_item_factory)
+	if (!cf->auto_resolution_ui_manager)
 		return;
 
-	w = gtk_item_factory_get_widget(cf->auto_resolution_item_factory,
-				_("/Control/Auto mode sticks at peak value"));
+	w = gtk_ui_manager_get_widget(cf->auto_resolution_ui_manager,
+                                      "/menubar/Control/AutoModeStickPeak");
 	GTK_CHECK_MENU_ITEM(w)->active = cf->auto_resolution_stick;
 
-	w = gtk_item_factory_get_widget(cf->auto_resolution_item_factory,
-				_("/Control/Auto mode recalibrate"));
+	w = gtk_ui_manager_get_widget(cf->auto_resolution_ui_manager,
+                                      "/menubar/Control/AutoModeRecalibrate");
 	if (cf->auto_grid_resolution)
 		gtk_widget_set_sensitive(w, TRUE);
 	else
@@ -2379,92 +2431,114 @@ cb_split_fraction(GtkWidget *adjustment, GkrellmChartdata *cd)
 /* =================================================================== */
 
 static void
-cb_auto_res_control(GkrellmChart *cp, guint option, GtkWidget* widget)
-    {
-    GkrellmChartconfig *cf;
-	gint			grid_res;
-	gboolean		active;
+cb_seq_control(GtkRadioAction *action, GtkRadioAction *current, GkrellmChart *cp )
+{
+    GkrellmChartconfig *cf = cp->config;
 
-	cf = cp->config;
-    switch (option)
-        {
-        case 0:
-			cp->maxval_auto_base = 0;
-			cp->maxval_peak = 0;
-			break;
-        case 1:
-            active = GTK_CHECK_MENU_ITEM(widget)->active;
-			cf->auto_resolution_stick = active;
-			cp->maxval_auto_base = 0;
-            break;
-        case 2:
-            active = GTK_CHECK_MENU_ITEM(widget)->active;
-			if (cf->sequence_125 && active)
-				return;
-			cf->sequence_125 = active;
-			break;
-        case 3:
-            active = GTK_CHECK_MENU_ITEM(widget)->active;
-			if (!cf->sequence_125 && active)
-				return;
-			cf->sequence_125 = !active;
+    if (cf->sequence_125 == gtk_radio_action_get_current_value(action))
+        return;
+    cf->sequence_125 = gtk_radio_action_get_current_value(action);
 
-			grid_res = gkrellm_125_sequence(cf->grid_resolution,
-						cf->sequence_125, cf->low, cf->high, TRUE, FALSE);
-			cf->grid_resolution = grid_res;
-			set_grid_resolution_spin_button(cp, grid_res);
-            break;
-        }
-	gkrellm_refresh_chart(cp);
-    }
+    cf->grid_resolution = gkrellm_125_sequence(cf->grid_resolution,
+                                               cf->sequence_125, cf->low, cf->high, TRUE, FALSE);
+    set_grid_resolution_spin_button(cp, cf->grid_resolution);
+}
 
+static void
+cb_auto_stick_control(GtkToggleAction *action, GkrellmChart *cp )
+{
+    GkrellmChartconfig *cf = cp->config;
 
-static GtkItemFactoryEntry	auto_res_control_items[] =
-    {
-{N_("/Control"),	NULL,	NULL,				 0,	"<LastBranch>" },
-{N_("/Control/-"),	NULL,	NULL,				 0,	"<Separator>"},
-{N_("/Control/Auto mode recalibrate"),
-					NULL,	cb_auto_res_control, 0, "<Item>"},
-{N_("/Control/Auto mode sticks at peak value"),
-					NULL,	cb_auto_res_control, 1,	"<ToggleItem>"},
-{N_("/Control/-"),	NULL,    NULL,				 0,	"<Separator>"},
-{N_("/Control/Sequence.../1 2 5"),
-					NULL,	cb_auto_res_control, 2,	"<RadioItem>"},
-{N_("/Control/Sequence.../1 1.5 2 3 5 7"),
-					NULL,	cb_auto_res_control, 3,
-					N_("/Control/Sequence.../1 2 5")},
-{N_("/Control/-"),	 NULL,	NULL,				 0,	"<Separator>"},
-    };
+    cf->auto_resolution_stick = gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
+    cp->maxval_auto_base = 0;
+    gkrellm_refresh_chart(cp);
+}
+
+static void
+cb_auto_res_control(GtkAction *action, GkrellmChart *cp )
+{
+    cp->maxval_auto_base = 0;
+    cp->maxval_peak = 0;
+    gkrellm_refresh_chart(cp);
+}
+
+static const char *auto_res_control_items = "\
+<ui>\
+  <menubar>\
+  <menu name=\"Control\" action=\"ControlAction\">\
+    <separator/>\
+    <menuitem name=\"AutoModeRecalibrate\" action=\"AutoModeRecalibrateAction\"/>\
+    <menuitem name=\"AutoModeStickPeak\" action=\"AutoModeStickPeakAction\"/>\
+    <menu name=\"SequenceMenu\" action=\"SequenceMenuAction\">\
+      <menuitem name=\"Seq125\" action=\"Seq125Action\"/>\
+      <menuitem name=\"Seq1357\" action=\"Seq1357Action\"/>\
+    </menu>\
+    <separator/>\
+  </menu>\
+  </menubar>\
+</ui>\
+";
+
+static GtkActionEntry auto_res_control_entries[] = 
+{
+    { "ControlAction", NULL, N_("Control"),
+      NULL, NULL, G_CALLBACK(NULL) },
+    { "SequenceMenuAction", NULL, N_("Sequence..."),
+      NULL, NULL, G_CALLBACK(NULL) },
+    { "AutoModeRecalibrateAction", NULL, N_("Auto mode recalibrate"),
+      NULL, NULL, G_CALLBACK(cb_auto_res_control) },
+};
+static guint n_auto_res_control_entries = G_N_ELEMENTS (auto_res_control_entries);
+
+static GtkToggleActionEntry auto_res_control_toggle_entries[] = 
+{
+    { "AutoModeStickPeakAction", NULL, N_("Auto mode sticks at peak value"),
+      NULL, NULL, G_CALLBACK(cb_auto_stick_control), FALSE },
+};
+static guint n_auto_res_control_toggle_entries = G_N_ELEMENTS (auto_res_control_toggle_entries);
+
+static GtkRadioActionEntry auto_res_control_radio_entries[] = 
+{
+    { "Seq125Action", NULL, N_("1 2 5"),
+      NULL, NULL, 1 },
+    { "Seq1357Action", NULL, N_("1 1.5 2 3 5 7"),
+      NULL, NULL, 0 },
+};
+static guint n_auto_res_control_radio_entries = G_N_ELEMENTS (auto_res_control_radio_entries);
 
 static void
 auto_resolution_control_menubar(GtkWidget **menubar, GkrellmChart *cp)
 	{
-	GtkItemFactory	*item_factory;
+        GtkUIManager *ui_manager;
+        GtkActionGroup *action_group;
 	GkrellmChartconfig		*cf = cp->config;
-	gint			i, n;
-	static gboolean	translated;
+        GError *error;
 
-	n = sizeof(auto_res_control_items) / sizeof(GtkItemFactoryEntry);
-	item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", NULL);
-	if (!translated)
-		{
-		for(i = 0; i < n; i++)
-			auto_res_control_items[i].path = _(auto_res_control_items[i].path);
-		auto_res_control_items[6].item_type =
-				_(auto_res_control_items[6].item_type);
-		translated = TRUE;
-		}
-	gtk_item_factory_create_items(item_factory, n, auto_res_control_items, cp);
-	cf->auto_resolution_item_factory = item_factory;
+        action_group = gtk_action_group_new ("ControlActions");
+        gtk_action_group_add_actions (action_group, auto_res_control_entries,
+                                      n_auto_res_control_entries, cp);
+        gtk_action_group_add_toggle_actions (action_group, auto_res_control_toggle_entries,
+                                             n_auto_res_control_toggle_entries, cp);
+        gtk_action_group_add_radio_actions (action_group, auto_res_control_radio_entries,
+                                            n_auto_res_control_radio_entries,
+                                            !!cf->sequence_125, G_CALLBACK(cb_seq_control),
+                                            cp);
+        ui_manager = gtk_ui_manager_new ();
+        error = NULL;
+        gtk_ui_manager_add_ui_from_string (ui_manager, auto_res_control_items,
+                                           strlen(auto_res_control_items), &error);
+        if (error)
+        {
+            g_message ("building menus failed: %s", error->message);
+            g_error_free (error);
+            return;
+        }
+        gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
+	cf->auto_resolution_ui_manager = ui_manager;
 	set_resolution_menubar_items_sensitivity(cf);
 
-	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(item_factory,
-		_("/Control/Sequence.../1 2 5")))->active = cf->sequence_125;
-	GTK_CHECK_MENU_ITEM(gtk_item_factory_get_widget(item_factory,
-		_("/Control/Sequence.../1 1.5 2 3 5 7")))->active = !cf->sequence_125;
-
 	if (menubar)
-		*menubar = gtk_item_factory_get_widget(item_factory, "<main>");
+		*menubar = gtk_ui_manager_get_widget(ui_manager, "/menubar");
 	}
 
 void
@@ -2551,7 +2625,7 @@ gkrellm_chartconfig_window_create(GkrellmChart *cp)
 		}
 
 	cf->auto_resolution_control_menubar = NULL;
-	cf->auto_resolution_item_factory = NULL;
+	cf->auto_resolution_ui_manager = NULL;
 	cf->grid_resolution_spin_button = NULL;
 	cf->fixed_grids_spin_button = NULL;
 
@@ -2703,20 +2777,20 @@ debug_dump_chart_list()
 	GkrellmPanel	*p;
 	GkrellmChartdata *cd;
 
-	printf("\n");
+	g_debug("\n");
 	for (list = gkrellm_get_chart_list(); list; list = list->next)
 		{
 		cp = (GkrellmChart *) list->data;
 		p = cp->panel;
 		if (p && p->label && p->label->string)
-			printf("%s [%d]: ", p->label->string, cp->style_id);
+			g_debug("%s [%d]: ", p->label->string, cp->style_id);
 		else
-			printf("(null) [%d]: ", cp->style_id);
+			g_debug("(null) [%d]: ", cp->style_id);
 		for (cdlist = cp->cd_list; cdlist; cdlist = cdlist->next)
 			{
 			cd = (GkrellmChartdata *) cdlist->data;
-			printf("%s %p->data ", cd->label, cd->data);
+			g_debug("%s %p->data ", cd->label, cd->data);
 			}
-		printf("\n");
+		g_debug("\n");
 		}
 	}
