@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 
@@ -24,6 +36,8 @@
 #include "gkrellm-private.h"
 #include "gkrellm-sysdeps.h"
 
+#include <inttypes.h>
+
 #if !defined(WIN32)
 #include <sys/socket.h>
 #include <utime.h>
@@ -32,8 +46,8 @@
 #include <arpa/inet.h>
 #include <netdb.h>
 #else
-#define uint32_t u_long
 #include <winsock2.h>
+#include <ws2tcpip.h>
 #endif
 
 #include <errno.h>
@@ -41,9 +55,11 @@
 
 
 #if defined(__linux__)
-#if defined(__GLIBC__) && ((__GLIBC__>2)||(__GLIBC__==2 && __GLIBC_MINOR__>=1))
 #define HAVE_GETADDRINFO	1
 #endif
+
+#if defined(__DragonFly__)
+#define HAVE_GETADDRINFO	1
 #endif
 
 #if defined(__FreeBSD__)
@@ -76,11 +92,15 @@ typedef struct
 	}
 	KeyTable;
 
+static gint		server_major_version,
+				server_minor_version,
+				server_rev_version;
+
 static gint		client_input_id,
 				client_fd;
 static gboolean	server_alive;
 
-static gchar	server_buf[4097];
+static gchar	server_buf[4097]; /* TODO: Use dynamic receive buffer */
 static gint		buf_index;
 
 static gchar	locale_decimal_point;
@@ -88,6 +108,20 @@ static gchar	server_decimal_point = '.';
 static gboolean	need_locale_fix = FALSE;
 
 
+gboolean
+gkrellm_client_check_server_version(gint major, gint minor, gint rev)
+    {
+    if (   server_major_version > major
+        || (server_major_version == major && server_minor_version > minor)
+        || (   server_major_version == major && server_minor_version == minor
+            && server_rev_version >= rev
+           )
+       )
+        return TRUE;
+    return FALSE;
+    }
+
+
   /* There can be a locale decimal point mismatch with the server.
   */
 static void
@@ -127,7 +161,7 @@ client_cpu_line_from_server(gchar *line)
 	gint	n;
 	guint64	user, nice, sys, idle;
 
-	sscanf(line, "%d %llu %llu %llu %llu", &n, &user, &nice, &sys, &idle);
+	sscanf(line, "%d %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64, &n, &user, &nice, &sys, &idle);
 	for (list = cpu_list; list; list = list->next)
 		{
 		cpu = (Cpu *) list->data;
@@ -250,10 +284,11 @@ client_sys_proc_init(void)
 /* ================================================================= */
 typedef struct
 	{
-	gchar	*name;
-	gchar	*subdisk_parent;
-	guint64	rblk,
-			wblk;
+	gchar		*name;
+	gchar		*subdisk_parent;
+	guint64		rblk,
+				wblk;
+	gboolean	virtual;
 	}
 	DiskData;
 
@@ -266,13 +301,26 @@ client_disk_line_from_server(gchar *line)
 	{
 	DiskData	*disk = NULL;
 	GList		*list;
-	gchar		name[16], s1[32], s2[32], s3[32];
+	gchar		*name;
+	gchar		s1[32], s2[32], s3[32];
 	guint64		rblk, wblk;
 	gint		n;
+	gboolean	virtual = FALSE;
 
-	n = sscanf(line, "%15s %31s %31s %31s", name, s1, s2, s3);
+	g_assert(line != NULL);
+	name = g_malloc(strlen(line) * sizeof(gchar));
+
+    /* FIXME: Extend client and server to support disks with both name and label */
+	n = sscanf(line, "%s %31s %31s %31s", name, s1, s2, s3);
 	if (n == 4)
 		{
+		if (   gkrellm_client_check_server_version(2, 2, 7)
+			&& !strcmp(s1, "virtual")
+		   )
+			{
+			virtual = TRUE;
+			s1[0] = '\0';
+			}
 		rblk = strtoull(s2, NULL, 0);
 		wblk = strtoull(s3, NULL, 0);
 		}
@@ -283,15 +331,22 @@ client_disk_line_from_server(gchar *line)
 		s1[0] = '\0';
 		}
 	else
+		{
+		g_warning("Invalid disk line received from server: \"%s\"", line);
+		g_free(name);
 		return;
+		}
+
 	for (list = disk_list; list; list = list->next)
 		{
+		/* Search disk in list */
 		disk = (DiskData *) list->data;
 		if (!strcmp(disk->name, name))
 			break;
 		}
 	if (!list)
 		{
+		/* Disk wasn't found, create new disk and add to list */
 		disk = g_new0(DiskData, 1);
 		disk->name = g_strdup(name);
 		if (s1[0])		/* I expect server to send in order */
@@ -300,9 +355,12 @@ client_disk_line_from_server(gchar *line)
 		}
 	if (disk)
 		{
+		/* Assign parsed data to disk */
 		disk->rblk = rblk;
 		disk->wblk = wblk;
+		disk->virtual = virtual;
 		}
+	g_free(name);
 	}
 
 static void
@@ -319,7 +377,7 @@ read_disk_data(void)
 					disk->subdisk_parent, disk->rblk, disk->wblk);
 		else
 			gkrellm_disk_assign_data_by_name(disk->name,
-					disk->rblk, disk->wblk);
+					disk->rblk, disk->wblk, disk->virtual);
 		}
 	}
 
@@ -338,7 +396,7 @@ client_sys_disk_init(void)
 
 	/* Disk monitor config needs to know will be using assign by name
 	*/
-	gkrellm_disk_assign_data_by_name(NULL, 0, 0);
+	gkrellm_disk_assign_data_by_name(NULL, 0, 0, FALSE);
 	}
 
 static void
@@ -354,12 +412,19 @@ typedef struct
 	gchar		*name;
 	gboolean	up_event,
 				down_event;
+
+	gint		up_time;
+
 	gulong		rx,
 				tx;
 	}
 	NetData;
 
-GList	*net_list;
+static NetData	*net_timer;
+
+static GList	*net_list;
+
+static gchar	*net_timer_name;
 
 static gboolean	net_server_use_routed;
 
@@ -371,7 +436,7 @@ client_net_line_from_server(gchar *line)
 	gchar		name[32];
 	guint64		rx, tx;
 
-	sscanf(line, "%31s %llu %llu", name, &rx, &tx);
+	sscanf(line, "%31s %" PRIu64 " %" PRIu64, name, &rx, &tx);
 	for (list = net_list; list; list = list->next)
 		{
 		net = (NetData *) list->data;
@@ -389,6 +454,9 @@ client_net_line_from_server(gchar *line)
 		net_list = g_list_append(net_list, net);
 		net->rx = rx;
 		net->tx = tx;
+
+		if (net_timer_name && !strcmp(net_timer_name, net->name))
+			net_timer = net;
 		}
 	}
 
@@ -422,6 +490,22 @@ client_net_routed_line_from_server(gchar *line)
 		}
 	}
 
+gint
+gkrellm_client_server_get_net_timer(void)
+	{
+	return net_timer ? net_timer->up_time : 0;
+	}
+
+static void
+client_net_timer_line_from_server(gchar *line)
+	{
+	gchar		name[32];
+
+	if (!net_timer)
+		return;
+	sscanf(line, "%s %d", name, &net_timer->up_time);
+	}
+
 static void
 check_net_routes(void)
 	{
@@ -458,16 +542,27 @@ client_sys_net_init(void)
 	{
 	gkrellm_net_client_divert(read_net_data, check_net_routes, NULL);
 	gkrellm_net_use_routed(net_server_use_routed);
+
+	if (net_timer_name)
+		gkrellm_net_server_has_timer();
 	}
 
 static void
 client_net_setup(gchar *line)
 	{
+	gchar	buf[32];
+
 	/* This is the server sysdep net_use_routed value.  The client <-> server
 	|  link always uses routed mode.
 	*/
 	if (!strcmp(line, "net_use_routed"))
 		net_server_use_routed = TRUE;
+
+	if (!strncmp(line, "net_timer", 9))
+		{
+		sscanf(line, "%*s %31s\n", buf);
+		net_timer_name = g_strdup(buf);
+		}
 	}
 
 
@@ -492,6 +587,8 @@ client_inet_line_from_server(gchar *line)
 #endif
 	gint		n, slen;
 
+	if (_GK.debug_level & DEBUG_INET)
+		g_print("inet server: %s\n", line);
 	if (*(line + 1) == '0')
 		{
 		n = sscanf(line + 3, "%x %127[^:]:%x", &tcp.local_port,
@@ -615,7 +712,7 @@ struct
 static void
 client_mem_line_from_server(gchar *line)
 	{
-	sscanf(line, "%llu %llu %llu %llu %llu %llu",
+	sscanf(line, "%" PRIu64 "%" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64,
 			&mem.total, &mem.used, &mem.free,
 			&mem.shared, &mem.buffers, &mem.cached);
 	}
@@ -623,7 +720,7 @@ client_mem_line_from_server(gchar *line)
 static void
 client_swap_line_from_server(gchar *line)
 	{
-	sscanf(line, "%llu %llu %lu %lu",
+	sscanf(line, "%" PRIu64 " %" PRIu64 " %lu %lu",
 			&mem.swap_total, &mem.swap_used,
 			&mem.swap_in, &mem.swap_out);
 	}
@@ -654,7 +751,7 @@ typedef struct
 	gchar	*directory,
 			*device,
 			*type;
-	gulong	blocks,
+	gint64	blocks,
 			bavail,
 			bfree,
 			bsize;
@@ -722,7 +819,9 @@ client_mounts_line_from_server(gchar *line)
 	else
 		{
 		m = g_new0(Mount, 1);
-		sscanf(line, "%127s %63s %63s %lu %lu %lu %lu", dir, dev, type,
+		sscanf(line,
+			"%127s %63s %63s %" PRIu64 " %" PRIu64 " %" PRIu64 " %" PRIu64,
+				dir, dev, type,
 				&m->blocks, &m->bavail, &m->bfree, &m->bsize);
 		m->directory = g_strdup(dir);
 		m->device = g_strdup(dev);
@@ -1206,6 +1305,13 @@ client_time_line_from_server(gchar *s)
 	}
 
 /* ================================================================= */
+static void
+client_server_version_setup(gchar *line)
+	{
+	sscanf(line, "%*s %d.%d.%d", &server_major_version,
+                    &server_minor_version, &server_rev_version);
+	}
+
 static void
 client_hostname_setup(gchar *s)
 	{
@@ -1312,6 +1418,18 @@ client_monitor_setup(gchar *line)
 static void
 client_server_error(gchar *line)
 	{
+	GtkWidget	*dlg;
+
+	dlg = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+				GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+				"GKrellM - connection refused from gkrellmd at %s:%d\n"
+				"\t%s\n\n"
+				"Edit gkrellmd.conf on %s to set:\n"
+				"\tmax-clents, allow-host, etc\n",
+				_GK.server, _GK.server_port, line, _GK.server);
+				gtk_dialog_run(GTK_DIALOG(dlg));
+	gtk_widget_destroy(dlg);
+
 	fprintf(stderr, "gkrellmd error: %s\n", line);
 	exit(0);
 	}
@@ -1355,6 +1473,7 @@ client_server_reconnect_timeout(gchar *line)
 /* ================================================================= */
 KeyTable	setup_table[] =
 	{
+	{"<version>",		client_server_version_setup },
 	{"<sensors_setup>",	client_sensors_setup },
 	{"<hostname>",		client_hostname_setup },
 	{"<sysname>",		client_sysname_setup },
@@ -1386,6 +1505,7 @@ typedef struct
 	ClientPlugin;
 
 static GList	*client_plugin_serve_data_list;
+static GList	*plugin_initial_update_list;
 
 static GkrellmFunc
 client_plugin_func(gchar *line)
@@ -1399,6 +1519,8 @@ client_plugin_func(gchar *line)
 	for (list = client_plugin_serve_data_list; list; list = list->next)
 		{
 		plug = (ClientPlugin *) list->data;
+		if (!plug->key_name)
+			continue;
 		n = strlen(plug->key_name);
 		s = line + n;
 		if (*s == '>' && !strncmp(plug->key_name, line, n))
@@ -1410,6 +1532,30 @@ client_plugin_func(gchar *line)
 	return func;
 	}
 
+static void
+client_plugin_initial_update(ClientPlugin *plug)
+	{
+	GList	*list;
+	void	(*func)(gchar *);
+	gchar	*line, *serve_name;
+
+	func = NULL;
+	serve_name = g_strdup_printf("<%s>", plug->key_name);
+	for (list = plugin_initial_update_list; list; list = list->next)
+		{
+		line = (gchar *) list->data;
+		if (*line == '<')
+			{
+			func = NULL;
+			if (!strcmp(line, serve_name))
+				func = plug->func_cb;
+			}
+		else if (func)
+			(*func)(line);
+		}
+	g_free(serve_name);
+	}
+
 void
 gkrellm_client_plugin_serve_data_connect(GkrellmMonitor *mon,
 			gchar *key_name, void (*func_cb)(gchar *line))
@@ -1424,6 +1570,8 @@ gkrellm_client_plugin_serve_data_connect(GkrellmMonitor *mon,
 	plug->func_cb = func_cb;
 	client_plugin_serve_data_list
 				= g_list_append(client_plugin_serve_data_list, plug);
+
+	client_plugin_initial_update(plug);
 	}
 
 static gboolean
@@ -1431,7 +1579,7 @@ client_send_to_server(gchar *buf)
 	{
 	gint	n;
 
-	if (!server_alive || client_fd < 0)
+	if (!server_alive || client_fd < 0 || !buf)
 		return FALSE;
 #if defined(MSG_NOSIGNAL)
 	n = send(client_fd, buf, strlen(buf), MSG_NOSIGNAL);
@@ -1440,8 +1588,8 @@ client_send_to_server(gchar *buf)
 #endif
 	if (n < 0 && errno == EPIPE)
 		{
-		if (_GK.debug_level & DEBUG_CLIENT)
-			printf("Write on closed pipe to gkrellmd server.\n");
+		gkrellm_debug(DEBUG_CLIENT, "Write on closed pipe to gkrellmd " \
+			"server, marking server-connection as dead\n");
 		server_alive = FALSE;
 		return FALSE;
 		}
@@ -1478,6 +1626,7 @@ KeyTable	update_table[] =
 	{"<disk>",			client_disk_line_from_server},
 	{"<net>",			client_net_line_from_server},
 	{"<net_routed>",	client_net_routed_line_from_server},
+	{"<net_timer>",		client_net_timer_line_from_server},
 	{"<mem>",			client_mem_line_from_server},
 	{"<swap>",			client_swap_line_from_server},
 	{"<fs>",			client_fs_line_from_server},
@@ -1496,8 +1645,9 @@ KeyTable	update_table[] =
 
 
 
+/* TODO: Port to GInputStream */
 static gint
-getline(gint fd, gchar *buf, gint len)
+gkrellm_getline(gint fd, gchar *buf, gint len)
 	{
 	fd_set			read_fds;
 	struct timeval	tv;
@@ -1525,11 +1675,10 @@ getline(gint fd, gchar *buf, gint len)
 		}
 	if (nread < 0 && errno != EINTR)
 		{
-		fprintf(stderr, "Broken server connection\n");
+		g_warning("Broken server connection\n");
 		exit(0);
 		}
-	if (_GK.debug_level & DEBUG_CLIENT)
-		printf("%s\n", buf);
+	gkrellm_debug(DEBUG_CLIENT, "%s\n", buf);
 	return n;
 	}
 
@@ -1563,17 +1712,24 @@ process_server_line(KeyTable *table, gint table_size, gchar *line)
 		}
 	else if (func)
 		(*func)(line);
+	if (!func && !setup_done)
+		plugin_initial_update_list
+				= g_list_append(plugin_initial_update_list, g_strdup(line));
 	}
 
 
   /* Read setup info from gkrellmd server.  Stuff needed before the
   |  client_init calls must be read here.
   */
-static void
+static gboolean
 read_server_setup(gint fd)
 	{
-	gchar			buf[256];
-	gint			table_size;
+	gchar	buf[4097]; /* TODO: Use dynamic receive buffer */
+	gint	table_size;
+	gint	rs;
+	gint	retries = 10;
+
+	gkrellm_debug(DEBUG_CLIENT, "read_server_setup()\n");
 
 	/* Pre 2.1.6 gkrellmd does not send <decimal_point>, so put a fallback
 	|  locale_sync() here.
@@ -1586,7 +1742,16 @@ read_server_setup(gint fd)
 
 	while (1)
 		{
-		getline(fd, buf, sizeof(buf));
+		rs = gkrellm_getline(fd, buf, sizeof(buf));
+		if (rs < 0)
+			return FALSE;
+		if (rs == 0)
+			{
+			if (--retries)
+				usleep(10000);
+			else
+				return FALSE;
+			}
 		if (!strcmp(buf, "</gkrellmd_setup>"))
 			break;
 		process_server_line(&setup_table[0], table_size, buf);
@@ -1602,19 +1767,32 @@ read_server_setup(gint fd)
 	table_size = sizeof(update_table) / sizeof(KeyTable);
 	while (1)
 		{
-		getline(fd, buf, sizeof(buf));
+		rs = gkrellm_getline(fd, buf, sizeof(buf));
+        if (rs < 0)
+            return FALSE;
+        if (rs==0){
+        	if(--retries)
+        		usleep(10000);
+        	else
+        		return FALSE;
+        }
 		if (!strcmp(buf, "</initial_update>"))
 			break;
 		process_server_line(&update_table[0], table_size, buf);
 		}
 	setup_done = TRUE;
+    return TRUE;
 	}
 
 void
 gkrellm_client_mode_disconnect(void)
 	{
 	if (client_fd >= 0)
+		{
+		gkrellm_debug(DEBUG_CLIENT, "gkrellm_client_mode_disconnect(); " \
+			"closing connection to server\n");
 		close(client_fd);
+		}
 	client_fd = -1;
 	server_alive = FALSE;
 	gdk_input_remove(client_input_id);
@@ -1631,6 +1809,8 @@ read_server_input(gpointer data, gint fd, GdkInputCondition condition)
 	count = recv(fd, server_buf + buf_index, n, 0);
 	if (count <= 0)
 		{
+		gkrellm_debug(DEBUG_CLIENT, "read_server_input(); recv() " \
+			"returned error, disconnecting from server\n", count);
 		gkrellm_client_mode_disconnect();
 		return;
 		}
@@ -1642,8 +1822,7 @@ read_server_input(gpointer data, gint fd, GdkInputCondition condition)
 	while (*line && (eol = strchr(line, '\n')) != NULL)
 		{
 		*eol = '\0';
-		if (_GK.debug_level & DEBUG_CLIENT)
-			printf("%s\n", line);
+		gkrellm_debug(DEBUG_CLIENT, "%s\n", line);
 		process_server_line(&update_table[0], table_size, line);
 		line = eol + 1;
 		}
@@ -1664,6 +1843,7 @@ read_server_input(gpointer data, gint fd, GdkInputCondition condition)
 	server_buf[buf_index] = '\0';
 	}
 
+
 gint
 gkrellm_connect_to(gchar *server, gint server_port)
 	{
@@ -1691,8 +1871,7 @@ gkrellm_connect_to(gchar *server, gint server_port)
 		if ((fd = socket(res->ai_family, res->ai_socktype,
 				res->ai_protocol)) < 0)
 			continue;
-		if (_GK.debug_level & DEBUG_CLIENT)
-			printf("\t[gkrellm_connect_to: (%d,%d,%d) %s:%d]\n",
+		gkrellm_debug(DEBUG_CLIENT, "\t[gkrellm_connect_to: (%d,%d,%d) %s:%d]\n",
 			       res->ai_family, res->ai_socktype,
 			       res->ai_protocol, server, server_port);
 		if (connect(fd, res->ai_addr, res->ai_addrlen) >= 0)
@@ -1706,8 +1885,8 @@ gkrellm_connect_to(gchar *server, gint server_port)
 		}
 	freeaddrinfo(res0);
 #else
-	if (_GK.debug_level & DEBUG_CLIENT)
-		printf("\t[gkrellm_connect_to: %s:%d]\n", server, server_port);
+	gkrellm_debug(DEBUG_CLIENT, "\t[gkrellm_connect_to: %s:%d]\n", server,
+		server_port);
 	addr = gethostbyname(server);
 	if (addr)
 		{
@@ -1715,11 +1894,12 @@ gkrellm_connect_to(gchar *server, gint server_port)
 		if (fd >= 0)
 			{
 			memset(&s, 0, sizeof(s));
-			memcpy(&s.sin_addr.s_addr, he->h_addr, he->h_length);
+			memcpy(&s.sin_addr.s_addr, addr->h_addr, addr->h_length);
 			s.sin_family = AF_INET;
 			s.sin_port = htons(server_port);
 			if (connect(fd, (struct sockaddr *)&s, sizeof (s)) < 0)
 				{
+				gkrellm_debug(DEBUG_CLIENT, "gkrellm_connect_to(); connect() failed\n");
 #ifdef WIN32
 				closesocket(fd);
 #else
@@ -1729,6 +1909,10 @@ gkrellm_connect_to(gchar *server, gint server_port)
 				}
 			}
 		}
+	else
+		{
+		gkrellm_debug(DEBUG_CLIENT, "gkrellm_connect_to(); gethostbyname() failed\n");
+		}
 #endif
 	if (fd < 0)
 		return -1;
@@ -1736,7 +1920,7 @@ gkrellm_connect_to(gchar *server, gint server_port)
 	return fd;
 	}
 
-gboolean
+enum GkrellmConnectResult
 gkrellm_client_mode_connect(void)
 	{
 	gchar	buf[128];
@@ -1745,10 +1929,8 @@ gkrellm_client_mode_connect(void)
 		_GK.server_port = GKRELLMD_SERVER_PORT;
 
 	client_fd = gkrellm_connect_to(_GK.server, _GK.server_port);
-	if (client_fd < 0) {
-		printf("%s\n", _("Unable to connect."));
-		return FALSE;
-	}
+	if (client_fd < 0)
+		return BAD_CONNECT;
 
 	snprintf(buf, sizeof(buf), "gkrellm %d.%d.%d%s\n",
 			GKRELLM_VERSION_MAJOR, GKRELLM_VERSION_MINOR,
@@ -1757,7 +1939,11 @@ gkrellm_client_mode_connect(void)
 
 	/* Initial setup lines from server are read in blocking mode.
 	*/
-	read_server_setup(client_fd);
+	if(!read_server_setup(client_fd))
+		{
+		close(client_fd);
+        return BAD_SETUP;
+    	}
 
 	/* Extra stuff not handled in read_server_setup()
 	*/
@@ -1765,29 +1951,31 @@ gkrellm_client_mode_connect(void)
 
 	/* Now switch to non blocking and set up a read handler.
 	*/
+#ifndef WIN32
 	fcntl(client_fd, F_SETFL, O_NONBLOCK);
+#endif
 	client_input_id = gdk_input_add(client_fd, GDK_INPUT_READ,
 					(GdkInputFunction) read_server_input, NULL);
 
 	server_alive = TRUE;
 
-	return TRUE;
+	return GOOD_CONNECT;
 	}
 
 
 static gboolean	client_mode_thread_busy;
 
-gint
+enum GkrellmConnectState
 gkrellm_client_server_connect_state(void)
 	{
 	if (client_mode_thread_busy)			/* reconnect in progress? */
-		return 2;
+		return CONNECTING;
 	if (_GK.client_mode && client_input_id > 0)	/* Currently connected? */
-		return 1;
+		return CONNECTED;
 	else if (_GK.client_mode)
-		return 0;
+		return DISCONNECTED;
 	else
-		return -1;
+		return NO_CLIENT;
 	}
 
 static gpointer
@@ -1801,10 +1989,13 @@ client_mode_connect_thread(void *data)
 void
 gkrellm_client_mode_connect_thread(void)
 	{
+	GThread	*gth;
+
 	if (client_mode_thread_busy || !_GK.client_mode)
 		return;
 	client_mode_thread_busy = TRUE;
-	g_thread_create(client_mode_connect_thread, NULL, FALSE, NULL);
+	gth = g_thread_new("client_mode_connect", client_mode_connect_thread,NULL);
+	g_thread_unref(gth);
 	}
 
 
