@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -155,9 +167,13 @@ gkrellm_theme_file_exists(char *name, gchar *subdir)
 			else
 				path = g_strdup_printf("%s/%s_%d%s", _GK.theme_path,
 						name, _GK.theme_alternative, image_type[i]);
-			if (   stat(path, &st) == 0
+#ifdef WIN32
+			if (g_stat(path, &st) == 0 && S_ISREG(st.st_mode))
+#else
+			if (   g_stat(path, &st) == 0
 				&& (S_ISREG(st.st_mode) || S_ISLNK(st.st_mode))
 			   )
+#endif
 				return path;
 			g_free(path);
 			path = NULL;
@@ -171,9 +187,13 @@ gkrellm_theme_file_exists(char *name, gchar *subdir)
 		else
 			path = g_strdup_printf("%s/%s%s", _GK.theme_path, name,
 					image_type[i]);
-		if (   stat(path, &st) == 0
+#ifdef WIN32
+			if (g_stat(path, &st) == 0 && S_ISREG(st.st_mode))
+#else
+		if (   g_stat(path, &st) == 0
 			&& (S_ISREG(st.st_mode) || S_ISLNK(st.st_mode))
 		   )
+#endif
 			return path;
 		g_free(path);
 		path = NULL;
@@ -249,7 +269,7 @@ assign_textcolor(GkrellmStyle *style, gchar *arg, gint AorB)
 	effect = gkrellm_cut_quoted_string(s, &s);
 	if (*color == '\0' || *shadowcolor == '\0' || *effect == '\0')
 		{
-		printf(_("Bad textcolor line %s\n"), arg);
+		g_warning(_("Bad textcolor line %s\n"), arg);
 		g_free(values);
 		return;
 		}
@@ -411,7 +431,7 @@ assign_style_entry(GkrellmStyle *style, gchar *value, gint entry_flag)
 		{
 		if (strcmp(value, "center") == 0)
 			style->label_position = GKRELLM_LABEL_CENTER;
-		else if (isdigit(*value))
+		else if (isdigit((unsigned char)*value))
 			style->label_position = atoi(value);
 		else
 			style->label_position = GKRELLM_LABEL_NONE;
@@ -491,7 +511,7 @@ assign_style(gchar *debug_name, GList *style_list, gint index,
 	if (index++ == 0)		/* style == style_list */
 		{
 		if (override)
-			printf("Bad override on DEFAULT: %s %s %d\n",
+			g_warning("Bad override on DEFAULT: %s %s %d\n",
 					debug_name, arg, entry_flag);
 		for (list = style_list->next; list; list = list->next, ++index)
 			{
@@ -547,7 +567,7 @@ assign_custom_style(gchar *debug_name, GList *style_list, gint index,
 		custom_style =
 				(GkrellmStyle *) g_list_nth_data(_GK.custom_style_list, i);
 
-//printf("assign_custom_style(%s, %s, %d, %s) %d\n",
+//g_debug("assign_custom_style(%s, %s, %d, %s) %d\n",
 //debug_name, custom_name, entry_flag, arg, i);
 	assign_style_entry(custom_style, arg, entry_flag);
 	set_themed(custom_style, entry_flag);
@@ -624,7 +644,7 @@ assign_gkrellmrc_style(gchar *source_line, gchar *area, gchar *string)
 		}
 	if (!mon_name || !entry || !*entry || !arg)
 		{
-		printf("StyleXXX ?: %s\n", source_line);
+		g_warning("StyleXXX ?: %s\n", source_line);
 		g_free(custom_name);
 		return;
 		}
@@ -647,7 +667,7 @@ assign_gkrellmrc_style(gchar *source_line, gchar *area, gchar *string)
 		}
 	else
 		{
-		printf("StyleXXX ?: %s\n", source_line);
+		g_warning("StyleXXX ?: %s\n", source_line);
 		g_free(custom_name);
 		return;
 		}
@@ -1029,13 +1049,13 @@ gkrellm_load_piximage_from_inline(gchar *name, const guint8 *data,
 		name = fname;
 		im = gkrellm_piximage_new_from_file(fname);
 		if (im == NULL)
-			printf(_("  Cannot load file image: %s\n"), fname);
+			g_warning(_("  Cannot load file image: %s\n"), fname);
 		}
 	if (im == NULL && data)
 		{
 		im = gkrellm_piximage_new_from_inline(data, copy_pixels);
 		if (im == NULL)
-			printf(_("  Cannot load GdkPixbuf inline data.\n"));
+			g_warning(_("  Cannot load GdkPixbuf inline data.\n"));
 		}
 	if (im && image)
 		{
@@ -1060,13 +1080,13 @@ gkrellm_load_piximage(gchar *name, gchar **xpm, GkrellmPiximage **image,
 		name = fname;
 		im = gkrellm_piximage_new_from_file(fname);
 		if (im == NULL)
-			printf(_("  Cannot load file image: %s\n"), fname);
+			g_warning(_("  Cannot load file image: %s\n"), fname);
 		}
 	if (im == NULL && xpm)
 		{
 		im = gkrellm_piximage_new_from_xpm_data(xpm);
 		if (im == NULL)
-			printf(_("  Cannot load xpm: %s\n"), name);
+			g_warning(_("  Cannot load xpm: %s\n"), name);
 		}
 	if (im && image)
 		{
@@ -1082,16 +1102,17 @@ load_from_piximage_list(gchar *name, GList *image_list,
 				gint index, gchar *subdir)
 	{
 	GList			*list;
-	GkrellmPiximage	**im;
+	GkrellmPiximage	*im;
 
 	list = g_list_nth(image_list, index);
 	if (list)
 		{
-		im = (GkrellmPiximage **) (&list->data);
-		gkrellm_load_piximage(name, NULL, im, subdir);
+		im = (GkrellmPiximage *) list->data;
+		gkrellm_load_piximage(name, NULL, &im, subdir);
+		list->data = (gpointer) im;
 		}
 	else
-		printf("Bad index %d for image list (meter/panel problem?)\n", index);
+		g_warning("Bad index %d for image list (meter/panel problem?)\n", index);
 	}
 
 static void
@@ -1204,7 +1225,7 @@ lookup_piximage_from_name(GList *image_list, gchar *name)
 static void
 load_piximage_table(ImageTable *ti, gint n_piximages, gchar *subdir)
 	{
-	GkrellmPiximage	**im;
+	GkrellmPiximage	*im;
 	GList			*list;
 	gint			i;
 
@@ -1217,8 +1238,9 @@ load_piximage_table(ImageTable *ti, gint n_piximages, gchar *subdir)
 							ti->name_in_list);
 			if (list)
 				{
-				im = (GkrellmPiximage **) (&list->data);
-				gkrellm_load_piximage(ti->name, ti->xpm, im, subdir);
+				im = (GkrellmPiximage *) list->data;
+				gkrellm_load_piximage(ti->name, ti->xpm, &im, subdir);
+				list->data = (gpointer) im;
 				}
 			}
 		else
@@ -1261,8 +1283,14 @@ destroy_piximage(GkrellmPiximage **im)
 static void
 destroy_piximage_list(GList *list, GList *name_list, gchar *debug_name)
 	{
+	GkrellmPiximage *im;
+
 	for ( ; list; list = list->next, name_list = name_list->next)
-		destroy_piximage((GkrellmPiximage **) (&list->data));
+		{
+		im = (GkrellmPiximage *) list->data;
+		destroy_piximage(&im);
+		list->data = NULL;
+		}
 	}
 
 static void
@@ -1275,7 +1303,7 @@ destroy_monitor_specific_piximages(void)
 		{
 		mon = (GkrellmMonitor *) list->data;
 		destroy_piximage(&mon->privat->top_spacer.piximage);
-		destroy_piximage(&mon->privat->bottom_spacer.piximage);	
+		destroy_piximage(&mon->privat->bottom_spacer.piximage);
 		}
 	}
 
@@ -1619,7 +1647,7 @@ gkrellm_get_gkrellmrc_string(gchar *string_name)
 	}
 
 static gboolean
-parse_monitor_config_keyword(gchar *line)
+parse_monitor_config_keyword(GkrellmMonitor *mon_only, gchar *line)
 	{
 	GList			*list;
 	GkrellmMonitor	*mon;
@@ -1636,6 +1664,8 @@ parse_monitor_config_keyword(gchar *line)
 	for (list = gkrellm_monitor_list; list; list = list->next)
 		{
 		mon = (GkrellmMonitor *) list->data;
+		if (mon_only && mon != mon_only)
+			continue;
 		if (!mon->config_keyword || strcmp(mon->config_keyword, keyword))
 			continue;
 		while (*line == ' ' || *line == '\t')
@@ -1720,12 +1750,12 @@ parse_config_line(gchar *line, struct _config *cf, gint size)
 			++s;
 		if (!*s || *s == '\n')
 			{
-			printf(_("Incomplete config line:\n    %s\n"), line);
+			g_warning(_("Incomplete config line:\n    %s\n"), line);
 			continue;
 			}
 		if (conf->value)
 			{
-			if (isdigit(*s) || *s == '-')
+			if (isdigit((unsigned char)*s) || *s == '-')
 				val = atoi(s);
 			else if (!strcmp(s, "on") || !strcmp(s, "true"))
 				val = 1;
@@ -2172,7 +2202,7 @@ parse_gkrellmrc(gint alternative)
 	if (alternative > 0)
 		snprintf(lbuf, sizeof(lbuf), "_%d", alternative);
 	path = g_strdup_printf("%s/%s%s", _GK.theme_path, GKRELLMRC, lbuf);
-	if ((f = fopen(path, "r")) != NULL)
+	if ((f = g_fopen(path, "r")) != NULL)
 		{
 		while (fgets(buf, sizeof(buf), f))
 			{
@@ -2219,9 +2249,10 @@ gkrellm_theme_config(void)
 	/* Warn theme developers!
 	*/
 	if (!_GK.use_top_bottom_margins && _GK.command_line_theme)
-		fprintf(stderr,
-				"Warning: Top and bottom meter/panel margins are not set.\n"
-				"         Do not depend on borders!!\n");
+		{
+		g_warning("Top and bottom meter/panel margins are not set.\n" \
+				" Do not depend on borders!\n");
+		}
 	}
 
 
@@ -2258,13 +2289,13 @@ struct	_config	user_config[] =
 
   /* The user_config is read twice.  Early to load _GK config values and then
   |  again when when building gkrellm for the first time to load monitor user
-  |  config values.
+  |  config values.  It's also read at plugin enables in case existing configs.
   */
 void
-gkrellm_load_user_config(gboolean monitor_values)
+gkrellm_load_user_config(GkrellmMonitor *mon_only, gboolean monitor_values)
 	{
 	FILE	*f;
-	gchar	*s, *ss, *config, *config_2_1_14 = NULL;
+	gchar	*s, *ss, *config;
 	gchar	buf[CFG_BUFSIZE];
 
 	if (!monitor_values)
@@ -2272,7 +2303,7 @@ gkrellm_load_user_config(gboolean monitor_values)
 		_GK.enable_hostname = TRUE;
 		_GK.hostname_short = FALSE;
 		_GK.enable_system_name = FALSE;
-		_GK.chart_width = 60;
+		_GK.chart_width = 100;
 		_GK.update_HZ = 10;
 		_GK.theme_scale = 100;
 		_GK.float_factor = 1.0;
@@ -2280,14 +2311,9 @@ gkrellm_load_user_config(gboolean monitor_values)
 		}
 	config = gkrellm_make_config_file_name(gkrellm_homedir(),
 				GKRELLM_USER_CONFIG);
-	f = fopen(config, "r");
+	f = g_fopen(config, "r");
+	g_free(config);
 
-	if (!f)		/* >= 2.1.15 configs are not backward compatible so there is */
-		{		/* a new config file name.  < 2.1.15 is forward compatible   */
-		config_2_1_14 = gkrellm_make_config_file_name(gkrellm_homedir(),
-					GKRELLM_2_1_14_CONFIG);
-		f = fopen(config_2_1_14, "r");
-		}
 	if (f)
 		{
 		while (fgets(buf, sizeof(buf), f))
@@ -2303,7 +2329,7 @@ gkrellm_load_user_config(gboolean monitor_values)
 			if (!strncmp(s, "float_factor ", 13))
 				sscanf(s + 13, "%f", &_GK.float_factor);
 			else if (monitor_values)
-				parse_monitor_config_keyword(s);
+				parse_monitor_config_keyword(mon_only, s);
 			else
 				parse_config_line(s, &user_config[0],
 						sizeof(user_config) / sizeof (struct _config));
@@ -2314,10 +2340,6 @@ gkrellm_load_user_config(gboolean monitor_values)
 		_GK.chart_width = CHART_WIDTH_MIN;
 	if (_GK.chart_width > CHART_WIDTH_MAX)
 		_GK.chart_width = CHART_WIDTH_MAX;
-//	if (config_2_1_14)
-//		rename(config_2_1_14, config);
-	g_free(config);
-	g_free(config_2_1_14);
 	}
 
 void
@@ -2397,24 +2419,28 @@ gkrellm_make_config_file_name(gchar *dir, gchar *config)
 void
 gkrellm_save_user_config(void)
 	{
-	FILE	*f;
-	GList	*list;
+	FILE			*f, *ff;
+	GList			*list;
 	GkrellmMonitor	*mon;
-	gint	i;
-	mode_t	mode;
-	gchar	*config;
+	gint			i;
+	mode_t			mode;
+	gchar			*config, *config_new;
 
-	if (_GK.demo)
+	if (_GK.demo || _GK.no_config)
 		return;
 	config = gkrellm_make_config_file_name(gkrellm_homedir(),
-								GKRELLM_USER_CONFIG);
-	f = fopen(config, "w");
-	g_free(config);
+					GKRELLM_USER_CONFIG);
+	config_new = g_strconcat(config, ".new", NULL);
+
+	f = g_fopen(config_new, "w");
 	if (f == NULL)
 		{
-		printf(_("Cannot open config file %s for writing.\n"), config);
+		g_warning(_("Cannot open config file %s for writing.\n"), config_new);
+		g_free(config_new);
+		g_free(config);
 		return;
 		}
+
 	fprintf(f,
 		"### GKrellM user config.  Auto written, do not edit (usually) ###\n");
 	fprintf(f, "### Version %d.%d.%d ###\n",
@@ -2439,6 +2465,69 @@ gkrellm_save_user_config(void)
 			gkrellm_record_state(INTERNAL, NULL);
 			}
 		}
+
+	if (   !_GK.config_clean
+	    && (ff = g_fopen(config, "r")) != NULL
+	   )
+		{
+		gchar		buf[CFG_BUFSIZE], *keyword, *ch, tmp;
+		struct _config
+					*cf,
+					*const end = user_config +
+								sizeof user_config / sizeof *user_config;
+
+		while (fgets(buf, sizeof(buf), ff))
+			{
+			for (ch = buf; *ch == ' ' || *ch == '\t'; ++ch)
+				;
+			if (*ch == '\n' || *ch == '#' || !*ch)
+				continue;
+			keyword = ch;
+			while (*ch && *ch != ' ' && *ch != '\t' && *ch != '\n')
+				++ch;
+			tmp = *ch;
+			*ch = 0;
+
+			for (list = gkrellm_monitor_list; list; list = list->next)
+				{
+				mon = (GkrellmMonitor *) list->data;
+				if (   mon->save_user_config && mon->privat->enabled
+				    && mon->config_keyword
+				    && !strcmp(mon->config_keyword, keyword)
+				   )
+					break;
+				}
+			if (list)
+				continue;
+
+			cf = user_config;
+			while (cf != end && strcmp(cf->option, keyword))
+				++cf;
+			if (cf != end)
+				continue;
+			if (!strcmp("float_factor", keyword))
+				continue;
+
+			*ch = tmp;
+			fputs(buf, f);
+			}
+		fclose(ff);
+		}
+	fclose(f);
+#if defined(WIN32)
+	/* windows rename() does not allow overwriting existing files! */
+	g_unlink(config);
+#endif
+	i = g_rename(config_new, config);
+	if (i != 0)
+		{
+		g_warning("Cannot rename new config file %s to %s.\n", config_new, config);
+		g_free(config);
+		g_free(config_new);
+		// NOTE: _GK.config_modified will stay true because saving failed
+		return;
+		}
+
 #if defined (S_IRUSR)
 	mode = (S_IRUSR | S_IWUSR);
 #elif defined (S_IREAD)
@@ -2446,9 +2535,10 @@ gkrellm_save_user_config(void)
 #else
 	mode = 0600;
 #endif
-	fchmod(fileno(f), mode);
-	fclose(f);
+	g_chmod(config, mode);
+
+	g_free(config);
+	g_free(config_new);
 
 	_GK.config_modified = FALSE;
 	}
-
