@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2009 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -49,7 +49,8 @@
 typedef struct
 	{
 	gchar			*name,
-					*label;			/* Possibly translated name */
+					*default_label,			/* Possibly translated name */
+					*label;
 	GtkWidget		*vbox;
 	GtkWidget		*enable_button;
 	GkrellmChart	*chart;
@@ -124,7 +125,7 @@ lookup_disk_by_device(gint major, gint minor)
 	}
 
 static DiskMon *
-lookup_disk_by_name(gchar *name)
+lookup_disk_by_name(const gchar *name)
 	{
 	DiskMon	*disk;
 	GList	*list;
@@ -139,19 +140,20 @@ lookup_disk_by_name(gchar *name)
 		|  written into the config.  XXX remove this eventually.
 		*/
 		if (   (disk == composite_disk || assign_method == DISK_ASSIGN_NTH)
-			&& !strcmp(name, disk->label))
+			&& !strcmp(name, disk->default_label))
 			return disk;
 		}
 	return NULL;
 	}
 
 static DiskMon *
-disk_new(gchar *name, gchar *label)
+disk_new(const gchar *name, const gchar *label)
 	{
 	DiskMon	*disk;
 
 	disk = g_new0(DiskMon, 1);
 	disk->name = g_strdup(name);
+	disk->default_label = g_strdup(label);
 	disk->label = g_strdup(label);
 	disk->launch.command = g_strdup("");
 	disk->launch.tooltip_comment = g_strdup("");
@@ -161,7 +163,7 @@ disk_new(gchar *name, gchar *label)
 	}
 
 static DiskMon *
-add_disk(gchar *name, gchar *label, gint major, gint minor, gint order)
+add_disk(const gchar *name, const gchar *label, gint major, gint minor, gint order)
 	{
 	DiskMon	*disk;
 	GList	*list;
@@ -190,7 +192,7 @@ add_disk(gchar *name, gchar *label, gint major, gint minor, gint order)
 	}
 
 static DiskMon *
-add_subdisk(gchar *subdisk_name, gchar *disk_name, gint subdisk)
+add_subdisk(const gchar *subdisk_name, const gchar *disk_name, gint subdisk)
 	{
 	DiskMon	*disk, *sdisk;
 	GList	*list = NULL;
@@ -275,6 +277,22 @@ gkrellm_disk_units_are_blocks(void)
 	units_are_blocks = TRUE;
 	}
 
+void
+gkrellm_disk_add_by_name(const gchar *name, const gchar *label)
+	{
+	gint	order = -1;
+
+	assign_method = DISK_ASSIGN_BY_NAME;
+	if (NULL == name) // Cannot add disk without a name
+		return;
+    if (order_from_name) // get optional order from sysdep-code
+        order = (*order_from_name)(name);
+    if (NULL != label) // label is optional and might be NULL
+        add_disk(name, label, 0, 0, order);
+    else
+        add_disk(name, name, 0, 0, order);
+	}
+
 void
 gkrellm_disk_assign_data_by_device(gint device_number, gint unit_number,
 			guint64 rb, guint64 wb, gboolean virtual)
@@ -305,8 +323,8 @@ gkrellm_disk_assign_data_nth(gint n, guint64 rb, guint64 wb, gboolean virtual)
 		disk = (DiskMon *) g_list_nth_data(disk_mon_list, n + 1);
 	else
 		{
-		sprintf(name, "%s%c", "Disk", 'A' + n);
-		sprintf(label, "%s%c", _("Disk"), 'A' + n);
+		snprintf(name, sizeof(name), "%s%c", "Disk", 'A' + n);
+		snprintf(label, sizeof(label), "%s%c", _("Disk"), 'A' + n);
 		disk = add_disk(name, label, 0, 0, n);
 		}
 	disk_assign_data(disk, rb, wb, virtual);
@@ -350,6 +368,8 @@ gkrellm_disk_subdisk_assign_data_by_name(gchar *subdisk_name, gchar *disk_name,
 		|  appended.  Eg. "hda1" is a subdisk_name of disk_name "hda"
 		*/
 		s = subdisk_name + strlen(disk_name);
+		if (*s == 'p')	/* except mmcblkN SD disks have "pN" partition numbers */
+			++s;
 		subdisk = strtol(s, &endptr, 0);
 		if (!*s || *endptr)
 			return;
@@ -709,7 +729,6 @@ destroy_disk_monitor(DiskMon *disk)
 	if (disk->launch.button)
 		gkrellm_destroy_button(disk->launch.button);
 	disk->launch.button = NULL;
-	disk->launch.tooltip = NULL;
 	gkrellm_dup_string(&disk->launch.command, "");
 	gkrellm_dup_string(&disk->launch.tooltip_comment, "");
 	gkrellm_chart_destroy(disk->chart);
@@ -927,11 +946,11 @@ save_disk_config(FILE *f)
 		have_enabled_subdisks = (disk->subdisk == -1) ?
 					any_enabled_subdisks(list->next) : FALSE;
 
-		fprintf(f, "%s device %s %d %d %d %d %d %d %d\n",
+		fprintf(f, "%s device %s %d %d %d %d %d %d %d %s\n",
 					DISK_CONFIG_KEYWORD,
 					disk->name, disk->major, disk->minor, disk->order,
 					disk->enabled, disk->extra_info, disk->subdisk,
-					have_enabled_subdisks);
+					have_enabled_subdisks, disk->label);
 		if (*(disk->launch.command) != '\0')
 			fprintf(f, "%s launch %s %s\n", DISK_CONFIG_KEYWORD,
 						disk->name, disk->launch.command);
@@ -957,7 +976,7 @@ load_disk_config(gchar *arg)
 	{
 	DiskMon		*disk = NULL;
 	gchar		config[32], item[CFG_BUFSIZE],
-				name[32], item1[CFG_BUFSIZE];
+				name[CFG_BUFSIZE], item1[CFG_BUFSIZE], disk_label[64];
 	gint		major, minor, enabled, extra, order, subdisk = -1;
 	gint		n;
 	gboolean	enabled_subdisks = TRUE;
@@ -971,7 +990,7 @@ load_disk_config(gchar *arg)
 			gkrellm_locale_dup_string(&text_format, item, &text_format_locale);
 		else if (!strcmp(config, "assign_method"))
 			sscanf(item, "%d", &config_assign_method);
-		else if (sscanf(item, "%31s %[^\n]", name, item1) == 2)
+		else if (sscanf(item, "%s %[^\n]", name, item1) == 2)
 			disk = lookup_disk_by_name(name);
 
 		if (!strcmp(config, "device"))
@@ -980,10 +999,10 @@ load_disk_config(gchar *arg)
 			|  if user changes kernel version.
 			*/
 			if (   config_assign_method == assign_method
-				&& sscanf(item1, "%d %d %d %d %d %d %d",
+				&& (n = sscanf(item1, "%d %d %d %d %d %d %d %63[^\n]",
 							&major, &minor, &order,
 							&enabled, &extra,
-							&subdisk, &enabled_subdisks) >= 5
+							&subdisk, &enabled_subdisks, disk_label)) >= 5
 			   )
 				{
 				/* A disk in the config may not have been found in the above
@@ -1006,6 +1025,8 @@ load_disk_config(gchar *arg)
 					{
 					disk->enabled = enabled;
 					disk->extra_info = extra;
+					if (n >= 8)
+						gkrellm_dup_string(&disk->label, disk_label);
 					}
 				}
 			return;
@@ -1037,11 +1058,14 @@ enum
 	{
 	NAME_COLUMN,
 	ENABLE_COLUMN,
+	LABEL_COLUMN,
 	DISK_COLUMN,
+	VISIBLE_COLUMN,
 	IMAGE_COLUMN,
 	N_COLUMNS
 	};
 
+static GtkTreeModel			*disk_model;
 static GtkTreeView			*treeview;
 
 static GtkTreeRowReference	*row_reference;
@@ -1065,7 +1089,9 @@ create_model(void)
 	tree = gtk_tree_store_new(N_COLUMNS,
 				G_TYPE_STRING,
                 G_TYPE_BOOLEAN,
+				G_TYPE_STRING,
 				G_TYPE_POINTER,
+				G_TYPE_BOOLEAN,
 				GDK_TYPE_PIXBUF);
 	for (list = disk_mon_list; list; )
 		{
@@ -1073,16 +1099,20 @@ create_model(void)
 		gtk_tree_store_append(tree, &iter, NULL);
 		if (list == disk_mon_list)
 			gtk_tree_store_set(tree, &iter,
-				NAME_COLUMN, _("Composite chart combines data for all disks"),
+				NAME_COLUMN, _("Composite chart"),
 				ENABLE_COLUMN, disk->enabled,
+				LABEL_COLUMN, disk->label,
 				DISK_COLUMN, disk,
+				VISIBLE_COLUMN, TRUE,
 				-1);
 		else
 			{
 			gtk_tree_store_set(tree, &iter,
-					NAME_COLUMN, disk->label,
+					NAME_COLUMN, disk->default_label,
 					ENABLE_COLUMN, disk->enabled,
+					LABEL_COLUMN, disk->label,
 					DISK_COLUMN, disk,
+					VISIBLE_COLUMN, TRUE,
 					-1);
 			if (disk->alert)
 				gtk_tree_store_set(tree, &iter,
@@ -1095,9 +1125,11 @@ create_model(void)
 				break;
 			gtk_tree_store_append(tree, &citer, &iter);
 			gtk_tree_store_set(tree, &citer,
-					NAME_COLUMN, disk->name,
+					NAME_COLUMN, disk->default_label,
 					ENABLE_COLUMN, disk->enabled,
+					LABEL_COLUMN, disk->label,
 					DISK_COLUMN, disk,
+					VISIBLE_COLUMN, TRUE,
 					-1);
 			if (disk->alert)
 				gtk_tree_store_set(tree, &citer,
@@ -1122,7 +1154,7 @@ add_launch_entry(GtkWidget *vbox, DiskMon *disk)
 	{
 	disk->launch_table = gkrellm_gtk_launcher_table_new(vbox, 1);
 	gkrellm_gtk_config_launcher(disk->launch_table, 0,  &disk->launch_entry,
-				&disk->tooltip_entry, disk->label,
+				&disk->tooltip_entry, disk->default_label,
 				&disk->launch);
 	g_signal_connect(G_OBJECT(disk->launch_entry), "changed",
 				G_CALLBACK(cb_launch_entry), disk);
@@ -1232,6 +1264,49 @@ cb_set_alert(GtkWidget *button, gpointer data)
 	gkrellm_alert_config_window(&disk->alert);
 	}
 
+static void
+label_edited_cb(GtkCellRendererText *cell, gchar *path_string,
+                gchar *new_label, gpointer data)
+	{
+	GtkTreeModel *model;
+	GtkTreeIter  iter;
+	GtkTreePath  *path;
+	DiskMon       *disk;
+
+	model = disk_model;
+	path = gtk_tree_path_new_from_string(path_string);
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_path_free(path);
+
+	gtk_tree_model_get(model, &iter,
+	                   DISK_COLUMN, &disk,
+	                   -1);
+	if (!*new_label)
+		new_label = disk->default_label;
+	gtk_tree_store_set(GTK_TREE_STORE(model), &iter,
+	                   LABEL_COLUMN, new_label, -1);
+
+	if (strcmp(new_label, disk->label))
+		{
+		gkrellm_dup_string(&disk->label, new_label);
+		gkrellm_panel_configure(disk->chart->panel, disk->label,
+				gkrellm_panel_style(style_id));
+		gkrellm_panel_create(disk->vbox, mon_disk, disk->chart->panel);
+		}
+#if 0
+	if (gkrellm_locale_dup_string(&s->name, new_label, &s->name_locale))
+		{
+		gkrellm_sensors_rebuild(s->type == SENSOR_TEMPERATURE,
+		                        s->type == SENSOR_FAN, s->type == SENSOR_VOLTAGE);
+		if (s->alert)
+			{
+			g_free(s->alert->name);
+			s->alert->name = g_strdup(s->name);
+//          gkrellm_reset_alert(s->alert);
+			}
+		}
+#endif
+	}
 
 static void
 cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
@@ -1250,7 +1325,7 @@ cb_tree_selection_changed(GtkTreeSelection *selection, gpointer data)
 	path = gtk_tree_model_get_path(model, &iter);
 	indices = gtk_tree_path_get_indices(path);
 	depth = gtk_tree_path_get_depth(path);
-// printf("selection: indices=[%d,%d]:%d, path=%s\n",
+// g_debug("selection: indices=[%d,%d]:%d, path=%s\n",
 //          indices[0], indices[1], gtk_tree_path_get_depth(path),
 //          gtk_tree_path_to_string(path));
 	change_row_reference(model, path);
@@ -1374,6 +1449,7 @@ create_disk_tab(GtkWidget *tab_vbox)
 	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
 
 	model = create_model();
+	disk_model = model;
 
 	treeview = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
 	gtk_tree_view_set_rules_hint(treeview, TRUE);
@@ -1391,6 +1467,15 @@ create_disk_tab(GtkWidget *tab_vbox)
 				renderer,
 				"text", NAME_COLUMN,
 				NULL);
+	renderer = gtk_cell_renderer_text_new();
+	gtk_tree_view_insert_column_with_attributes(treeview, -1, _("Label"),
+				renderer,
+				"text", LABEL_COLUMN,
+				"editable", TRUE,
+				"visible", VISIBLE_COLUMN,
+				NULL);
+	g_signal_connect(G_OBJECT(renderer), "edited",
+				G_CALLBACK(label_edited_cb), NULL);
 
 	renderer = gtk_cell_renderer_pixbuf_new();
 	gtk_tree_view_insert_column_with_attributes(treeview, -1, "",
@@ -1436,7 +1521,7 @@ create_disk_tab(GtkWidget *tab_vbox)
 	vbox1 = gkrellm_gtk_category_vbox(vbox,
 				_("Launch Commands"),
 				4, 0, TRUE);
-	launch_vbox = gkrellm_gtk_scrolled_vbox(vbox, NULL,
+	launch_vbox = gkrellm_gtk_scrolled_vbox(vbox1, NULL,
 						GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_widget_show(launch_vbox);
 	gtk_widget_realize(launch_vbox);
