@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -17,6 +17,18 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 /* These functions define the GKrellM interface to the system dependent
@@ -47,6 +59,16 @@
 void	gkrellm_sys_main_init(void);
 void	gkrellm_sys_main_cleanup(void);
 
+	/* If the sysdep code needs to call a function for additional
+	|  setup after the main() arguments are parsed and GTK is
+	|  initialized, register the function by calling this from
+	|  gkrellm_sys_main_init().
+	*/
+void	gkrellm_sys_setup_connect(void (*setup_func)());
+
+	/* The setup function will be passed the main() argc and argv and
+	|  should be declared like:  void XXX_setup(gint ac, gchar **av);
+	*/
 
 /* ===================================================================== */
 /* Misc monitor interface functions */
@@ -61,11 +83,7 @@ gint	gkrellm_get_timer_ticks(void);
   */
 void	gkrellm_mail_local_unsupported(void);
 
-#if GLIB_CHECK_VERSION(2,0,0)
 GThread *gkrellm_mail_get_active_thread(void);
-#else
-gpointer gkrellm_mail_get_active_thread(void);
-#endif
 
 /* ===================================================================== */
 /* CPU monitor interface
@@ -136,10 +154,11 @@ gboolean gkrellm_sys_disk_init(void);
 void	gkrellm_sys_disk_read_data(void);
 gchar	*gkrellm_sys_disk_name_from_device(gint device_number,
 				gint unit_number, gint *order);
-gint	gkrellm_sys_disk_order_from_name(gchar *name);
+gint	gkrellm_sys_disk_order_from_name(const gchar *name);
 
 void	gkrellm_disk_client_divert(void (*read_func)(),
 			gchar *(*name_from_device_func)(), gint (*order_from_name_func)());
+void    gkrellm_disk_add_by_name(const gchar *name, const gchar *label);
 void	gkrellm_disk_assign_data_by_device(gint device_number,
 				gint unit_number, guint64 r, guint64 w, gboolean virtual);
 void	gkrellm_disk_assign_data_nth(gint n, guint64 r, guint64 w,
@@ -157,7 +176,7 @@ void	gkrellm_disk_units_are_blocks(void);	/* Otherwise bytes */
 /* Inet monitor interface
 |
 |  System dependent code should #include "inet.h" and inet.h
-|  should #define INET6 if IPV6 protocal is available.
+|  should #define INET6 if IPV6 protocol is available.
 */
 
 gboolean gkrellm_sys_inet_init(void);
@@ -238,7 +257,7 @@ void	gkrellm_fs_client_divert(void (*get_fsusage_func)(),
 				void (*get_mounts_func)(), void (*get_fstab_func)(),
 				gboolean (*fstab_modified_func)());
 void	gkrellm_fs_assign_fsusage_data(gpointer fs,
-				glong blocks, glong avail, glong free, glong bsize);
+				gint64 blocks, gint64 avail, gint64 free, gint64 bsize);
 void	gkrellm_fs_add_to_fstab_list(gchar *dir, gchar *dev, gchar *type,
 				gchar *options);
 void	gkrellm_fs_add_to_mounts_list(gchar *dir, gchar *dev, gchar *type);
