@@ -1,5 +1,5 @@
 /* GKrellM
-|  Copyright (C) 1999-2010 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
 |  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
@@ -88,7 +88,7 @@ static GtkWidget *top_window;
 GList			*gkrellm_monitor_list;
 time_t			gkrellm_time_now;
 
-static GtkItemFactory	*item_factory;
+static GtkUIManager	*ui_manager;
 
 static gchar	*geometry;
 
@@ -401,17 +401,17 @@ update_monitors()
 void
 gkrellm_start_timer(gint Hz)
 	{
-	static gint	timeout_id	= 0;
+	static guint	timeout_id	= 0;
 	gint		interval;
 
 	if (timeout_id)
-		gtk_timeout_remove(timeout_id);
+		g_source_remove(timeout_id);
 	timeout_id = 0;
 	if (Hz > 0)
 		{
 		interval = 1000 / Hz;
 		interval = interval * 60 / 63;	/* Compensate for overhead XXX */
-		timeout_id = gtk_timeout_add(interval,
+		timeout_id = g_timeout_add(interval,
 						(GtkFunction) update_monitors,NULL);
 		}
 	}
@@ -602,7 +602,7 @@ gkrellm_motion(GtkWidget *widget, GdkEventMotion *ev, gpointer data)
 void
 gkrellm_menu_popup(void)
 	{
-	gtk_menu_popup(GTK_MENU(item_factory->widget), NULL, NULL, NULL, NULL,
+            gtk_menu_popup(GTK_MENU(gtk_ui_manager_get_widget(ui_manager, "/popup")), NULL, NULL, NULL, NULL,
 					0, gtk_get_current_event_time());
 	}
 
@@ -639,7 +639,7 @@ top_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 
 	if (ev->button == 3)
 		{
-		gtk_menu_popup(GTK_MENU(item_factory->widget), NULL, NULL, NULL, NULL,
+		gtk_menu_popup(GTK_MENU(gtk_ui_manager_get_widget(ui_manager, "/popup")), NULL, NULL, NULL, NULL,
 					ev->button, ev->time);
 		return FALSE;
 		}
@@ -755,7 +755,7 @@ side_frame_button_press(GtkWidget *widget, GdkEventButton *ev, gpointer data)
 
 	if (ev->button == 3)
 		{
-		gtk_menu_popup(GTK_MENU(item_factory->widget), NULL, NULL, NULL, NULL,
+		gtk_menu_popup(GTK_MENU(gtk_ui_manager_get_widget(ui_manager, "/popup")), NULL, NULL, NULL, NULL,
 					ev->button, ev->time);
 		return FALSE;
 		}
@@ -979,11 +979,10 @@ draw_left_frame_overlap(GdkPixbuf *pixbuf, GkrellmBorder *border,
 	GkrellmPiximage		piximage;
 	GdkPixmap			*pixmap = NULL;
 	GdkBitmap			*mask = NULL;
-	gint				w_pixbuf, h_pixbuf;
+	gint				h_pixbuf;
 
 	if (overlap <= 0)
 		return;
-	w_pixbuf = gdk_pixbuf_get_width(pixbuf);
 	h_pixbuf = gdk_pixbuf_get_height(pixbuf);
 	piximage.pixbuf = gdk_pixbuf_new_subpixbuf(pixbuf, 0, 0,
 						overlap, h_pixbuf);
@@ -1554,9 +1553,10 @@ create_widget_tree()
 
 	gtk_widget_realize(gtree.window);
 
-	/* gtk_window_set_resizable() */
-	g_object_set(G_OBJECT(gtree.window), "allow_shrink", FALSE, NULL);
-	g_object_set(G_OBJECT(gtree.window), "allow_grow", FALSE, NULL);
+	/* Set the toplevel window size handling to be under program control.
+	*/
+	gtk_window_set_resizable((GtkWindow *) gtree.window, FALSE);
+
 	if (!decorated)
 		gtk_window_set_decorated((GtkWindow *) gtree.window, FALSE);
 
@@ -1674,7 +1674,7 @@ cb_configure_notify(GtkWidget *widget, GdkEventConfigure *ev, gpointer data)
 
 	/* At startup can be a race for configure events to be reporting correct
 	|  move to position and pack_side_frames() first call.  Initial configure
-	|  events will be intermediate x,y values as intial mapping happens, so
+	|  events will be intermediate x,y values as initial mapping happens, so
 	|  using configure event position values is delayed.
 	*/
 	if (!configure_position_lock)
@@ -2091,7 +2091,6 @@ main(gint argc, gchar **argv)
 #ifdef ENABLE_NLS
 	gtk_set_locale();
 #endif
-	g_thread_init(NULL);
 	gtk_init(&argc, &argv);		/* Will call gdk_init() */
 	gkrellm_log_init();
 	gtk_widget_push_colormap(gdk_rgb_get_colormap());
@@ -2324,7 +2323,7 @@ main(gint argc, gchar **argv)
 	g_signal_connect(G_OBJECT(gtree.right_event_box), "button_press_event",
 				G_CALLBACK(side_frame_button_press), NULL );
 
-	item_factory = gkrellm_create_item_factory_popup();
+	ui_manager = gkrellm_create_ui_manager_popup();
 
 	if (_GK.sticky_state)
 		gtk_window_stick(GTK_WINDOW(top_window));
