@@ -1,22 +1,34 @@
 /* GKrellM
-|  Copyright (C) 1999-2004 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
 |
-|  Author:  Bill Wilson    bill@gkrellm.net
+|  Author:  Bill Wilson    billw@gkrellm.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
-|  This program is free software which I release under the GNU General Public
-|  License. You may redistribute and/or modify this program under the terms
-|  of that license as published by the Free Software Foundation; either
-|  version 2 of the License, or (at your option) any later version.
 |
-|  This program is distributed in the hope that it will be useful,
-|  but WITHOUT ANY WARRANTY; without even the implied warranty of
-|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-|  GNU General Public License for more details.  Version 2 is in the
-|  COPYRIGHT file in the top level directory of this distribution.
-| 
-|  To get a copy of the GNU General Puplic License, write to the Free Software
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+|  GKrellM is free software: you can redistribute it and/or modify it
+|  under the terms of the GNU General Public License as published by
+|  the Free Software Foundation, either version 3 of the License, or
+|  (at your option) any later version.
+|
+|  GKrellM is distributed in the hope that it will be useful, but WITHOUT
+|  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+|  or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public
+|  License for more details.
+|
+|  You should have received a copy of the GNU General Public License
+|  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
 
 #include "gkrellm.h"
@@ -102,7 +114,7 @@ gkrellm_cut_quoted_string(gchar *src, gchar **endptr)
 			{
 			if (endptr)
 				*endptr = src;
-			printf(_("Unterminated quote\n"));
+			g_warning(_("Unterminated quote\n"));
 			return NULL;
 			}
 		*s = '\0';
@@ -217,8 +229,8 @@ gkrellm_make_home_subdir(gchar *subdir, gchar **path)
 	dir = g_build_path(G_DIR_SEPARATOR_S, gkrellm_homedir(), subdir, NULL);
 	if (!g_file_test(dir, G_FILE_TEST_IS_DIR))
 		{
-		if (mkdir(dir, 0755) < 0)
-			printf(_("Cannot create directory: %s\n"), dir);
+		if (g_mkdir(dir, 0755) < 0)
+			g_warning(_("Cannot create directory: %s\n"), dir);
 		else
 			result = TRUE;
 		}
@@ -235,12 +247,15 @@ gkrellm_make_home_subdir(gchar *subdir, gchar **path)
 gchar *
 gkrellm_gtk_entry_get_text(GtkWidget **entry)
 	{
-	gchar	*s = "";
+	static /*const*/ gchar *def_s = "";
+	gchar *s = def_s;
 
 	if (*entry)
-		s = (gchar *) gtk_entry_get_text(GTK_ENTRY(*entry));
-	while (*s == ' ' || *s == '\t')
-		++s;
+		{
+		s = (gchar *)gtk_entry_get_text(GTK_ENTRY(*entry));
+		while (*s == ' ' || *s == '\t')
+			++s;
+		}
 	return s;
 	}
 
@@ -287,7 +302,7 @@ gkrellm_gdk_text_width(PangoFontDescription *font_desc,
 	pango_layout_set_text(layout, string, len);
 	pango_layout_get_pixel_size(layout, &w, &h);
 	g_object_unref(layout);
-	return w;	
+	return w;
 	}
 
 gint
@@ -302,7 +317,7 @@ gkrellm_gdk_text_markup_width(PangoFontDescription *font_desc,
 	pango_layout_set_markup(layout, string, len);
 	pango_layout_get_pixel_size(layout, &w, &h);
 	g_object_unref(layout);
-	return w;	
+	return w;
 	}
 
 gint
@@ -512,3 +527,8 @@ gkrellm_locale_dup_string(gchar **dst_utf8, gchar *src, gchar **dst_locale)
 	return TRUE;
 	}
 
+guint
+big_endian_uint(guint8 *b)
+	{
+	return ((b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3]);
+	}
