@@ -1,7 +1,9 @@
 /* GKrellM
-|  Copyright (C) 1999-2007 Bill Wilson
+|  Copyright (C) 1999-2014 Bill Wilson
+|                2007-2014 Stefan Gehn
 |
-|  Author:  Bill Wilson    billw@gkrellm.net
+|  Authors:  Bill Wilson    billw@gkrellm.net
+|            Stefan Gehn    stefan+gkrellm@srcbox.net
 |  Latest versions might be found at:  http://gkrellm.net
 |
 |
@@ -17,9 +19,20 @@
 |
 |  You should have received a copy of the GNU General Public License
 |  along with this program. If not, see http://www.gnu.org/licenses/
+|
+|
+|  Additional permission under GNU GPL version 3 section 7
+|
+|  If you modify this program, or any covered work, by linking or
+|  combining it with the OpenSSL project's OpenSSL library (or a
+|  modified version of that library), containing parts covered by
+|  the terms of the OpenSSL or SSLeay licenses, you are granted
+|  additional permission to convey the resulting work.
+|  Corresponding Source for a non-source form of such a combination
+|  shall include the source code for the parts of OpenSSL used as well
+|  as that of the covered work.
 */
-|  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-*/
+
 
 #include "win32-plugin.h"
 
@@ -71,7 +84,7 @@ void win32_init_callbacks(void)
     gkrellm_callbacks.gkrellm_add_chartdata = gkrellm_add_chartdata;
     gkrellm_callbacks.gkrellm_add_default_chartdata = gkrellm_add_default_chartdata;
     gkrellm_callbacks.gkrellm_alloc_chartdata = gkrellm_alloc_chartdata;
-    gkrellm_callbacks.gkrellm_store_chartdata = gkrellm_store_chartdata;
+    gkrellm_callbacks.gkrellm_store_chartdatav = gkrellm_store_chartdatav;
     gkrellm_callbacks.gkrellm_draw_chartdata = gkrellm_draw_chartdata;
     gkrellm_callbacks.gkrellm_monotonic_chartdata = gkrellm_monotonic_chartdata;
     gkrellm_callbacks.gkrellm_get_chartdata_hide = gkrellm_get_chartdata_hide;
@@ -436,5 +449,11 @@ void win32_init_callbacks(void)
     gkrellm_callbacks.gkrellm_gdk_text_markup_width = gkrellm_gdk_text_markup_width;
     gkrellm_callbacks.gkrellm_gdk_draw_string_markup = gkrellm_gdk_draw_string_markup;
     gkrellm_callbacks.gkrellm_gdk_draw_text_markup = gkrellm_gdk_draw_text_markup;
+
+    //---------------------------------------------------------------------------
+    // new since 2.3.2
+
+    gkrellm_callbacks.gkrellm_debugv = gkrellm_debugv;
+
 } // win32_init_callbacks()
 
